@@ -117,7 +117,6 @@ class FeedFoward(nn.Module):
             self.non_linearity = nn.Tanh()
         if hidden_size is None or hidden_size == 0:
             hidden_size = input_size * 4
-        self.rec = False
         if dropout is not None and dropout != 0:
             self.net = nn.Sequential(
                 nn.Linear(input_size, hidden_size),
@@ -126,26 +125,14 @@ class FeedFoward(nn.Module):
                 nn.Dropout(dropout),
             )
         else:
-            if non_linearity == "LSTM":
-                self.net = nn.LSTM(
-                    input_size, hidden_size, num_layers=2, batch_first=True
-                )
-                self.c = torch.randn(1, hidden_size)
-                self.h = torch.randn(1, hidden_size)
-                self.rec = True
-            else:
-                self.net = nn.Sequential(
-                    nn.Linear(input_size, hidden_size),
-                    self.non_linearity,
-                    nn.Linear(hidden_size, input_size),
-                )
+            self.net = nn.Sequential(
+                nn.Linear(input_size, hidden_size),
+                self.non_linearity,
+                nn.Linear(hidden_size, input_size),
+            )
 
     def forward(self, x):
-        if self.rec is True:
-            out, (self.h, self.c) = self.net(x, (self.h, self.c))
-            return out
-        else:
-            return self.net(x)
+        return self.net(x)
 
 
 class Block(nn.Module):
@@ -316,8 +303,9 @@ class MultiHeadSelfAttention(nn.Module):
     def generate(self, idx, max_new_tokens, temperature=1.0, top_k=None):
         """Generate new tokens given a context
 
-        Note: for apple MPS, top_k is limited max 16! (Current (01/2023) implementation limitation)
+        Note: for apple MPS, top_k is limited max 16 vor older torchs! ((01/2023) implementation limitation)
         See: https://github.com/pytorch/pytorch/issues/78915
+        Solved in: https://github.com/pytorch/pytorch/pull/94639 (03/2023)
 
         :param idx: the context (B,T) tensor of indices
         :param max_new_tokens: the maximum number of tokens to generate
